// Library		libSOCK - Machine independent Socket I/O functions.
// File:		SOCK_HAddrSI.c
// Function:	SOCK_HAddrSI
// Descrizione:	Restituisce l'indirizzo IP dell' host indicato in forma
//				ASCII puntata e integer in network byte order nei parametri
//				indirizzati da puntatori non nulli.
//				Return: 0 e indirizzo caricato, se l'host esiste; -1 e
//						parametri di uscita invariata se l'host non esiste
//						o se sono stati rilevati altri errori di basso li-
//						vello.
// Author(s):	TECMAV srl - av/cb
// L.M. Date:	17-Jun-2003 (02:44:55)

#include "_libsock.h"
#include "libDPR.h"

int SOCK_HAddrSI(char *hostn, char *ip_str, int *ip_int)
{
	union
	{
		int i;
		struct
		{
			unsigned char a;
			unsigned char b;
			unsigned char c;
			unsigned char d;
		}b;
	}
	u1;

	struct hostent 		*h;
	char *p1, *p2;	


	// If hostname is an IP address, convert it directly.

	for(u1.i = 0, p1 = hostn, p2 = &u1.b.a; *p1; p1++)
		if ('0' <= *p1 && *p1 <= '9')
			*p2 = (*p2 * 10) + *p1 - '0';
		else
		{
			if (*p1 != '.') break;
			if (p2 == (char *)&u1.b.d) break;
			p2++;
		}

	if (*p1)
	{
		// hostn is not an IP address. Searches by name.

    	h = gethostbyname (hostn);

		DPR3("gethostbyname(%s)=%d - errno=%d\n", hostn, (int)h, errno)

    	if (!h) 
		{
			errno = BAD_HOSTNAME;
			return (-1);
		}
    	u1.i = *(int *)(*(h->h_addr_list));
	}


	// Lo standard di rete e` 68K (MSb a indirizzo basso)

	if (ip_str)
		sprintf(ip_str, "%d.%d.%d.%d", u1.b.a, u1.b.b, u1.b.c, u1.b.d);

	if (ip_int) *ip_int = u1.i;

	return(0);
}
