// Library		libSOCK - Machine independent Socket I/O functions.
// File:		SOCK_Peer.c
// Function:	SOCK_Peer
// Descrizione: Apre un socket UDP verso hostn/port:
//				- hostn = Nome (/etc/hosts) o IP address in char.
//				- portn = qualunque (anche < 1024) numero di porta.
//				Return:
//				- ret. value = 0/-1 OK/not OK
//				- struttura "peer" inizializzata (selo se ret. value == 0);
// Author(s):	TECMAV srl - av
// L.M. Date:	11-Dec-2003 (03:16:02)

#include "_libsock.h"
#include "libDPR.h"


#define ABAN	{ DPR0("] -> -1\n") return(-1); }

int SOCK_Peer(char *hostn, int portn, SockPeer_t *peer)
{
	struct sockaddr_in	*soc;
	int					 sd, rt;


	DPR2("[ SOCK_Peer(hostn=\"%s\", portn=%d)\n", hostn, portn)


	memset (peer, 0, sizeof(SockPeer_t));
	errno = 0;

	sd = socket(AF_INET, SOCK_DGRAM, 0);
	DPR3("socket()=%d - errno=%d=%s\n", sd, errno, CML_EStr())
	if (sd < 0) { close(sd); ABAN }

	rt = SOCK_HAddrSI(hostn, peer->s_ip, &(peer->i_ip));
	DPR6("SOCK_HAddrSI(%s,%s,0x%X)=%d  (errno=%d=%s)\n",
		hostn, peer->s_ip, peer->i_ip, rt, errno, CML_EStr())
	if (rt < 0){ close(sd); ABAN }

	soc = (struct sockaddr_in *)(peer->soc);
	SET_ADDR_LEN(((struct sockaddr_in)(peer->soc)))
	soc->sin_family	= AF_INET;

	rt = bind(sd,(struct sockaddr *)soc, sizeof(struct sockaddr_in));
	DPR5("bind(sd=%d,..., siz=%d)=%d (errno=%d=%s)\n",
            sd, (int)sizeof(struct sockaddr_in), rt, errno, CML_EStr())
	if (rt < 0){ close(sd); ABAN }

	soc->sin_port		 = htons(portn);
	soc->sin_addr.s_addr = peer->i_ip;
	DPR_SOC(soc)

	peer->sd = sd;
	DPR0("] -> OK=0\n")
	return (0);
}
