// Library		libSOCK - Machine independent Socket I/O functions.
// File:		SOCK_Recv.c
// Function:	SOCK_Recv
// Descrizione: Legge un messaggio da "peer" mediante recfrom().
//				Restituisce il numero di caratteri letti o -1.
// Author(s):	TECMAV srl - av
// L.M. Date:	18-Dec-2003 (17:00:21)

#include "_libsock.h"
#include "libDPR.h"

static int sig_alrm_flag;

static CML_SIG_HANDLER(sig_alrm_handler) { sig_alrm_flag = 1; }

int SOCK_Recv(SockPeer_t *peer, char *msg, int size, int tout)
{
	CMLsighp_t prev;			// Ptr. a precedente SIGALRM handler.
	int letti;					// Byte letti ad ogni read.
	int siz;					// Parametro per rcvfrom.


	DPR3("[ SOCK_Recv(peer=0x%X, msg=0x%x, tout=%d)\n",
		(int)peer, (int)msg, tout)

	errno = sig_alrm_flag = 0;			// Se fossero sporchi ...

	siz = sizeof(struct sockaddr_in);

	if (tout < 0) tout = 0;

	if(tout) 											// Set timeout.
	{
		prev = CML_Signal(SIGALRM, sig_alrm_handler);
		alarm(tout);
	}

	letti = recvfrom(	peer->sd,
						msg,
						(size_t)size,
						0,
						(struct sockaddr *)(peer->soc),
						&siz
					);

	if(tout)
	{
		if (!sig_alrm_flag) alarm(0);
		CML_Signal(SIGALRM, prev);
	}


	DPR4("] ret=%d, errno=%d=%s,  sig_alrm_flag=%d\n",
		letti, errno, CML_EStr(), sig_alrm_flag)

	return(letti);
}
